#include <pthread.h>
#include <semaphore.h>
#include <stdio.h>
#include <stdlib.h>

#define MAX 10000000

// Ici, on protège l'accès au compteur avec l'algorithme d'exclusion mutuelle
// de Peterson. Bien que l'algorithme soit correct, le compteur final sera
// légèrement moins que 2*MAX. En fait, l'implémentation ci-dessous ne garantit
// pas toujours l'exclusion mutuelle, en raison des «optimisations» faites
// au niveau du processeur qui peut échanger l'ordre des read et write
// considérés comme indépendant.

char f[2],v;
int counter = 0;

void* count(void* data)
{
	int i = (long int)data;
	int autre = 1-i;

	int k;

	for (k = 0; k < MAX; k++)
	{
		f[i] = 1;	// algorithme de Peterson
		v = i;
		while (f[autre] && v == i);

		counter++;

		f[i] = 0;	// on quitte la section critique
	}
	return NULL;
}

int main(int argc, char * argv[])
{
	f[0] = f[1] = v = 0;

	pthread_t t1;
	pthread_t t2;

	pthread_create(&t1, NULL, count, (void*)0);  // create first thread
	pthread_create(&t2, NULL, count, (void*)1);  // create second thread

	pthread_join(t1, NULL);  // wait for first thread
	pthread_join(t2, NULL);  // wait for second thread

	printf("Counter: %d\n", counter);
	return 0;
}
